
package com.honeywell.samples.iqimage;

import com.honeywell.iqimagemanager.IQConstValue;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Rect;
import android.util.AttributeSet;
import android.widget.ImageView;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * Displays image from engine and device. The ImagingView class can load images
 * from various sources (captured image, previewing frame, user selected) and
 * can scale it by Zoombar.
 */
public class IQImageView extends ImageView implements IConstValue, IQConstValue {
    private Bitmap mIQBitmap = null;

    private int mCurrentType = 0;

    private boolean isNeedShowIQ = false;

    private SimpleDateFormat timeStampFormat = new SimpleDateFormat("MMddHHmmssSS");

    public IQImageView(Context context) {
        super(context);
        setFocusable(true);

    }

    public IQImageView(Context context, AttributeSet attrs) {
        super(context, attrs);
        setFocusable(true);
    }

    public IQImageView(Context context, AttributeSet attrs, int defStyle) {
        super(context);
        setFocusable(true);
    }

    public void setIQType(int type) {
        mCurrentType = type;
        initializeIQBtimap();
    }

    private boolean initializeIQBtimap() {
        int width;
        int height;
        switch (mCurrentType) {
            case IQTYPE_DELIVERY:

                width = DELIVERY_WIDTH * IQ_RESOLUTION;
                height = DELIVERY_HEIGHT * IQ_RESOLUTION;
                break;
            case IQTYPE_POSTAL:
                width = POSTAL_WIDTH * IQ_RESOLUTION;
                height = POSTAL_HEIGHT * IQ_RESOLUTION;
                break;
            default:
                return false;
        }
        return creatIQBitmap(width, height);
    }

    public Bitmap getIQBitmap() {
        return mIQBitmap;
    }

    public boolean creatIQBitmap(int width, int height) {
        boolean isNeed = false;
        if (mIQBitmap == null)
            isNeed = true;
        if (mIQBitmap != null)
            if (mIQBitmap.isRecycled())
                isNeed = true;
        if (isNeed) {
            mIQBitmap = Bitmap.createBitmap(width, height, Bitmap.Config.RGB_565);
            if (mIQBitmap == null)
                return false;
            return true;
        } else
            return false;

    }

    public void releaseBitmap() {
        if (mIQBitmap != null) {
            mIQBitmap.recycle();
            mIQBitmap = null;
        }
    }

    public void setBitmapFlag(boolean flag) {
        isNeedShowIQ = flag;
    }

    public void setBitmap(Bitmap bitmap) {
        mIQBitmap = bitmap;
    }

    /**
     * Saves captured image into a file with specified path.
     * 
     * @param filename The string object contains the absolute path. If filename
     *            is NULL, try to save captured image into a file with an
     *            appointed file path.
     * @return Whether the procedure of saving image is successful.
     */
    public boolean saveBitmap(String filename, int imagetype) {
        String type;
        switch (imagetype) {
            case IQTYPE_DELIVERY:
                type = "delivery_";
                break;
            case IQTYPE_POSTAL:
                type = "postal_";
                break;
            case IQTYPE_EMBED:
                type = "embed_";
                break;
            case IQTYPE_NOFORMAT:
                type = "noformat_";
                break;
            default:
                type = "others_";
                break;
        }
        String image_name;
        boolean resultcompress = false;

        if (filename == "") {
            image_name = STING_IQ_FOLER + "/" + type + timeStampFormat.format(new Date()) + ".png";
        }

        else {
            image_name = filename;
        }

        OutputStream outStream = null;

        try {

            outStream = new FileOutputStream(image_name);
            resultcompress = mIQBitmap.compress(Bitmap.CompressFormat.PNG, 100, outStream);
            outStream.flush();
            outStream.close();

        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return resultcompress;
    }

    public boolean saveBarcode(String barcode, int format) {
        boolean resutl = true;
        String type = "";
        switch (format) {
            case IQTYPE_DELIVERY:
                type = "delivery";
                break;
            case IQTYPE_POSTAL:
                type = "postal";
                break;
            case IQTYPE_EMBED:
                type = "embed";
                break;
            case IQTYPE_NOFORMAT:
                type = "noformat";
                break;
            default:
                type = "others";
                break;
        }
        String filename = STING_IQ_FOLER + "/" + type + timeStampFormat.format(new Date()) + ".txt";
        try {
            File f = new File(filename);
            BufferedWriter output = new BufferedWriter(new FileWriter(f));

            output.write(barcode);
            output.flush();
            output.close();
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
        return resutl;
    }

    @Override
    protected void onDraw(Canvas canvas) {
        if (!isNeedShowIQ)
            return;
        if (mIQBitmap == null)
            return;
        int areaw = getWidth() - 10;
        int areah = getHeight() - 10;
        // setBackgroundColor(Color.BLACK);
        int bmpw = mIQBitmap.getWidth();
        int bmph = mIQBitmap.getHeight();
        int left = getPaddingLeft() + 5;
        int top = this.getPaddingTop() + 5;
        boolean isneedScale = ((areaw < bmpw) || (areah < bmph)) ? true : false;

        if (mIQBitmap != null) {
            Rect mSrc = new Rect();
            mSrc.set(0, 0, bmpw, bmph);
            Rect mDst = new Rect();
            if (isneedScale) {
                int scaledh = 0;
                int scaledw = 0;
                if ((areaw * bmph) >= areah * bmpw) {
                    scaledh = areah;
                    scaledw = bmpw * scaledh / bmph;
                } else {
                    scaledw = areaw;
                    scaledh = bmph * scaledw / bmpw;
                }
                mDst.set(left + (areaw - scaledw) / 2, top + (areah - scaledh) / 2, scaledw,
                        scaledh);
            } else
                mDst.set(left + (areaw - bmpw) / 2, top + (areah - bmph) / 2, bmpw, bmph);
            canvas.drawBitmap(mIQBitmap, mSrc, mDst, null);
        }

    }

}
