
package com.honeywell.samples.iqimage;

import com.honeywell.decodemanager.barcode.DecodeResult;
import com.honeywell.decodemanager.barcode.IQBitmapParcel;
import com.honeywell.decodemanager.barcode.IQImagingProperties;
import com.honeywell.iqimagemanager.IQConstValue;
import com.honeywell.iqimagemanager.IQImageManager;

import android.app.TabActivity;
import android.content.Context;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.os.RemoteException;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.RelativeLayout;
import android.widget.TabHost;
import android.widget.TextView;

import java.io.File;

/**
 * IQImagingSample supports Delivery and Postal IQ image
 */
public class IQImagingSample extends TabActivity implements IConstValue, IQConstValue,
        TabHost.TabContentFactory, TabHost.OnTabChangeListener {
    private static final String TAG = "IQDemo";

    // private IDecoderService mDecoderServiceEngineType;
    private boolean mokayToDecode = true;

    private boolean okayToSaveRaw = true;

    private boolean okayToSavePng = true;

    protected static final int RES_LABEL_ITEM = android.R.layout.simple_list_item_1;

    private IQImagingProperties mProperty;

    private IQImageManager mIQManger;

    private int mIQType = IQTYPE_DELIVERY;

    // layout inflater object used to inflate views
    private IQImageView mIQImageView;

    private View mLinearView = null;

    private EditText tracEdit;

    private EditText addressEditv;

    private Context mContext;

    private TextView mAspectRatioText = null;

    private TextView mFormatText = null;

    private TextView mXText = null;

    private TextView mYText = null;

    private TextView mWText = null;

    private TextView mHText = null;

    private int mImageType = IMAGE_BINARY;

    ArrayAdapter<String> mSpinnerAdapter;

    private final int SCANKEY = 0x94;
    
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.iqtab);
        mContext = this;
        initTabHost();
        // first is to initialize delivery tab
        View mRootView = this.getTabHost().getTabContentView();
        mLinearView = mRootView.findViewById(R.id.deliverylayout);
        tracEdit = (EditText) mLinearView.findViewById(R.id.delivery_tracking);
        addressEditv = (EditText) mLinearView.findViewById(R.id.delivery_address);

        mProperty = new IQImagingProperties();
        mProperty.Reserved = -1;
        // tv = (TextView) findViewById(R.id.ScanResult);
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        Log.d(TAG, "onKeyDown enter");
        switch (keyCode) {
            case KeyEvent.KEYCODE_G:
                if (okayToSaveRaw) {
                    okayToSaveRaw = false;
                }
                return true;
            case KeyEvent.KEYCODE_J:
                if (okayToSavePng) {
                    okayToSavePng = false;
                }
                return true;
            case KeyEvent.KEYCODE_BACK:
                this.finish();
                return true;
                
            case KeyEvent.KEYCODE_UNKNOWN:	
            	if(event.getScanCode() == SCANKEY || event.getScanCode() == 87 || event.getScanCode() == 88) {
                    if (mokayToDecode == true) {
                        mokayToDecode = false;
                        scanBarcode();
                    }
            	}
            	return true;
            default:
                return super.onKeyDown(keyCode, event);
        }
    }

    @Override
    public boolean onKeyUp(int keyCode, KeyEvent event) {
        Log.d(TAG, "onKeyDown enter");
        switch (keyCode) {
            case KeyEvent.KEYCODE_G:
                okayToSaveRaw = true;
                return true;
            case KeyEvent.KEYCODE_J:
                okayToSavePng = true;
                return true;
            case KeyEvent.KEYCODE_BACK:
                this.finish();
                return true;                
            case KeyEvent.KEYCODE_UNKNOWN:	
            	if(event.getScanCode() == SCANKEY  || event.getScanCode() == 87 || event.getScanCode() == 88) 
                    mokayToDecode = true;
            	return true;
            default:
                return super.onKeyUp(keyCode, event);
        }
    }

    @Override
    public void onResume() {
        super.onResume();
        File file = new File(STING_IQ_FOLER);
        if (!file.isDirectory()) {
            file.mkdir();
        }
        mIQManger = new IQImageManager(this, mHandler);
        mIQManger.initIQEnv();
    }

    @Override
    public void onPause() {
        super.onPause();
        mIQManger.reaseIQEnv();
    }

    @Override
    public void onDestroy() {
        // unbindDecoderService();
        super.onDestroy();
        if (mIQImageView != null) {
            mIQImageView.releaseBitmap();
            mIQImageView = null;
        }
    }

    private Handler mHandler = new Handler() {

        public void handleMessage(Message msg) {
            getIQImage();
            switch (msg.what) {
                case MESSAGE_DECODE_SUC:

                    DecodeResult decRes = (DecodeResult) msg.obj;
                    if (decRes.length != 0) {
                        switch (mIQType) {

                            case IQTYPE_DELIVERY:
                                ParseDeliveryBarCode(decRes.barcodeData);
                                configureProperties(mIQType, decRes.barcodeData);

                                break;
                            case IQTYPE_POSTAL:
                                configureProperties(mIQType, decRes.barcodeData);
                                tracEdit.setText(decRes.barcodeData);

                                break;
                            default:
                                configureProperties(IQTYPE_DEFAULT, decRes.barcodeData);
                                break;
                        }

                        if (mIQImageView != null) {
                            Log.i(TAG, "mIQBitMap's configure = "
                                    + mIQImageView.getIQBitmap().getConfig().toString());
                            Log.i(TAG, "width = " + mIQImageView.getIQBitmap().getWidth());
                            IQBitmapParcel bmpparcel = new IQBitmapParcel(
                                    mIQImageView.getIQBitmap());
                            int ress = -1;
                            try {
                                ress = mIQManger.getIQImage(mProperty, bmpparcel);
                            } catch (RemoteException e) {
                                // TODO Auto-generated catch block
                                e.printStackTrace();
                            }
                            if (ress == 0) {
                                // release first, than change to updated bitmap
                                // mIQImageView.releaseBitmap();
                                mIQImageView.setBitmap(bmpparcel.getBitmap());
                                Log.d(TAG, "Get IQ image successfully!!!!");
                                mIQImageView.setBitmapFlag(true);
                                mIQImageView.invalidate();
                            } else {
                                Log.d(TAG, "No IQ IMAGE !!!! ret= " + ress);

                            }
                        } else
                            Log.d(TAG, "Init IQImage error!!!!!!");

                    }
                    break;

                default:
                    break;
            }
        }
    };

    private void initTabHost() {

        final TabHost tabHost = getTabHost();
        tabHost.addTab(tabHost.newTabSpec(TAB_DELIVERY).setIndicator(getString(R.string.delivery))
                .setContent(R.id.deliverylayout));
        tabHost.addTab(tabHost.newTabSpec(TAB_POSTAL).setIndicator(getString(R.string.postal))
                .setContent(R.id.postallayout));

        tabHost.setCurrentTabByTag(TAB_DELIVERY);
        tabHost.setOnTabChangedListener(this);
        tabHost.setup(getLocalActivityManager());

        int totalTabs = tabHost.getTabWidget().getChildCount();
        for (int i = 0; i < totalTabs; i++) {
            ((TextView) ((RelativeLayout) tabHost.getTabWidget().getChildTabViewAt(i))
                    .getChildAt(1)).setHeight(30);
            ((TextView) ((RelativeLayout) tabHost.getTabWidget().getChildTabViewAt(i))
                    .getChildAt(1)).setTextAppearance(mContext,
                    android.R.style.TextAppearance_Medium);
            tabHost.getTabWidget().getChildAt(i).getLayoutParams().height = 50;
        }

    }

    private void getIQImage() {

        switch (mIQType) {
            case IQTYPE_DELIVERY: {
                if (mIQImageView == null) {
                    if (mLinearView != null)
                        mIQImageView = (IQImageView) mLinearView.findViewById(R.id.deli_img);
                }
            }
                break;
            case IQTYPE_POSTAL:
                if (mLinearView != null)
                    mIQImageView = (IQImageView) mLinearView.findViewById(R.id.postal_img);
                break;

            default:

                break;
        }
        if (mIQImageView != null)
            mIQImageView.setIQType(mIQType);
    }

    private void clearView() {
        if (tracEdit != null)
            tracEdit.setText("");
        if (addressEditv != null)
            addressEditv.setText("");
        if (mIQImageView != null)
            mIQImageView.setBitmapFlag(false);

        if (mAspectRatioText != null)
            mAspectRatioText.setText("");
        if (mFormatText != null)
            mFormatText.setText("");
        if (mXText != null)
            mXText.setText("");
        if (mYText != null)
            mYText.setText("");
        if (mWText != null)
            mWText.setText("");
        if (mHText != null)
            mHText.setText("");
    }

    private void scanBarcode() {
        DecodeResult decRes = new DecodeResult();
        clearView();
        try {
            mIQManger.decodeIQWithTimout(1000, decRes);
        } catch (RemoteException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

    }

    private int configureProperties(int type, String barcodeData) {
        int ret = 0;
        mProperty.Format = (mImageType == IMAGE_BINARY) ? 0 : 1;
        switch (type) {
            case IQTYPE_DELIVERY:
                mProperty.AspectRatio = 1;
                mProperty.X_Offset = 2;
                mProperty.Y_Offset = 44;
                mProperty.Width = 124;
                mProperty.Height = 58;
                mProperty.Resolution = 4;

                break;
            case IQTYPE_POSTAL:
                mProperty.AspectRatio = 23;
                mProperty.X_Offset = -8;
                mProperty.Y_Offset = -58;
                mProperty.Width = 180;
                mProperty.Height = 90;
                mProperty.Resolution = 4;
                break;
            default:
                mProperty.AspectRatio = 1;
                mProperty.X_Offset = 2;
                mProperty.Y_Offset = 44;
                mProperty.Width = 124;
                mProperty.Height = 58;
                mProperty.Resolution = 4;
                break;
        }
        return ret;
    }

    private void ParseDeliveryBarCode(String strBarCode) {
        String tracing = "";
        String address = "";
        char[] chStr = strBarCode.toCharArray();
        int parapos = 0;
        while (parapos < strBarCode.length()) {
            if ((chStr[parapos] != '\r') && (chStr[parapos] != '\n'))
                ++parapos;
            else {
                tracing = strBarCode.substring(0, parapos);
                break;
            }
        }
        while ((parapos < strBarCode.length())
                && ((chStr[parapos] == '\r') || (chStr[parapos] == '\n')))
            ++parapos;

        if (parapos < strBarCode.length())
            address = strBarCode.substring(parapos, strBarCode.length() - 1);
        if (mLinearView != null) {

            if (tracEdit != null)
                tracEdit.setText(tracing);

            if (addressEditv != null)
                addressEditv.setText(address);
        }

    }

    public void onTabChanged(String tabId) {
        // TODO Auto-generated method stub
        View mRootView = this.getTabHost().getTabContentView();
        clearView();

        tracEdit = null;
        addressEditv = null;
        mLinearView = null;
        mAspectRatioText = null;
        mFormatText = null;
        mXText = null;
        mYText = null;
        mWText = null;
        mHText = null;

        if (TAB_DELIVERY.equalsIgnoreCase(tabId)) {
            mIQType = IQTYPE_DELIVERY;
            mLinearView = mRootView.findViewById(R.id.deliverylayout);
            if (mLinearView != null) {
                tracEdit = (EditText) mLinearView.findViewById(R.id.delivery_tracking);
                addressEditv = (EditText) mLinearView.findViewById(R.id.delivery_address);
            }

        } else if (TAB_POSTAL.equalsIgnoreCase(tabId)) {
            mLinearView = mRootView.findViewById(R.id.postallayout);
            mIQType = IQTYPE_POSTAL;
            if (mLinearView != null) {
                tracEdit = (EditText) mLinearView.findViewById(R.id.postal_tracking);
            }
        }
        // release bitmap of last tab
        if (mIQImageView != null) {
            mIQImageView.releaseBitmap();
            mIQImageView = null;
        }

    }

    public View createTabContent(String tag) {
        // TODO Auto-generated method stub
        return null;
    }

}
