
package com.honeywell.samples.imagingsample;

import com.honeywell.imagingmanager.ImageConst;
import com.honeywell.imagingmanager.ImagingManager;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.os.Handler;
import android.util.AttributeSet;
import android.widget.ImageView;

/**
 * Displays image from engine and device. The ImagingView class can load images
 * from various sources (captured image, previewing frame, user selected) and
 * can scale it by Zoombar.
 */
public class ImagingView extends ImageView implements ImageConst {

    private Bitmap mBitmap = null;

    private Bitmap mPreviewBitmap = null;

    private Bitmap mImageFileBitmap = null;

    private boolean isNeedshowBitmap = false;

    private boolean isPreviewing = false;

    private boolean isNeedShowImagefile = false;

    private boolean isNeedUpdateLayout = false;

    private int mOrginWidth = 0;

    private int mOriginHeight = 0;

    private int mScaleWidht = 0;

    private int mScaleHeight = 0;

    private int mMinWidth = 0;

    private int mMaxWidth = 0;

    private int mWidhtStep = 0;

    private int mHeightStep = 0;

    private int mZoomNum = 0;

    private Handler mHandler = null;

    public ImagingView(Context context) {
        super(context);
        setFocusable(true);
        // mOrginWidth = mScaleWidht = getWidth();
        // mOriginHeight = mScaleHeight = getHeight();
//        mBitmap = Bitmap.createBitmap(CAPTUERED_IMAGE_WIDTH, CAPTUERED_IMAGE_HEIGHT,
//                Bitmap.Config.RGB_565);
//        mPreviewBitmap = Bitmap.createBitmap(CAPTUERED_IMAGE_WIDTH / PREVIEW_RATIO,
//                CAPTUERED_IMAGE_HEIGHT / PREVIEW_RATIO, Bitmap.Config.ALPHA_8);

    }

    public ImagingView(Context context, AttributeSet attrs) {
        super(context, attrs);
        setFocusable(true);
        // mOrginWidth = mScaleWidht = getWidth();
        // mOriginHeight = mScaleHeight = getHeight();
//        mBitmap = Bitmap.createBitmap(CAPTUERED_IMAGE_WIDTH, CAPTUERED_IMAGE_HEIGHT,
//                Bitmap.Config.RGB_565);
//        mPreviewBitmap = Bitmap.createBitmap(CAPTUERED_IMAGE_WIDTH / PREVIEW_RATIO,
//                CAPTUERED_IMAGE_HEIGHT / PREVIEW_RATIO, Bitmap.Config.ALPHA_8);

    }

    public ImagingView(Context context, AttributeSet attrs, int defStyle) {
        super(context);
        setFocusable(true);
        // mOrginWidth = mScaleWidht = getWidth();
        // mOriginHeight = mScaleHeight = getHeight();
    }

    public void initBitmap(int w ,int h){
    	if (mBitmap == null)
    		mBitmap = Bitmap.createBitmap(w, h,Bitmap.Config.RGB_565);
    	if (mPreviewBitmap == null)
    		mPreviewBitmap = Bitmap.createBitmap(w / PREVIEW_RATIO,
    				h / PREVIEW_RATIO, Bitmap.Config.ALPHA_8);
    }

    /**
     * Reset the parameters of imaging layout when showing mode is changed . For
     * example ,user-selected picture ==> captured image , user-selected picture
     * ==> previewing image, etc.
     */
    private void initCaptureParametre() {
        mZoomNum = 0;
        mMinWidth = getWidth() / 4;
        mMaxWidth = getWidth() * 2;
        mWidhtStep = getWidth() * 1000 / 40;
        mHeightStep = getHeight() * 1000 / 40;
        mOrginWidth = mScaleWidht = getWidth();
        mOriginHeight = mScaleHeight = getHeight();
    }

    /**
     * Set status the flag of captured image.
     * 
     * @param ison Whether to show captured image.
     */
    public void setShowBitmapflag(boolean ison) {
        if (ison) {
            initCaptureParametre();
        }
        isNeedshowBitmap = ison;
    }

    /**
     * Gets the flag of the captured image .
     * 
     * @return Returns the status of captured image.
     */
    public boolean isNeedShowBitmap() {
        return isNeedshowBitmap;
    }

    /**
     * Set the update flag. This method is called when shows new captured
     * image/previewing frame after Zoombar's taking effect on last image.
     * 
     * @param isUpdate whether to re-calculate the layout parameter.
     */
    public void setUpdateFlag(boolean isUpdate) {
        isNeedUpdateLayout = isUpdate;
    }

    /**
     * Set the flag of previewing status.
     * 
     * @param ispreview Whether to enter previewing mode.
     */
    public void setPreviewFlag(boolean ispreview) {
        if (!isPreviewing && ispreview)
            initCaptureParametre();
        isPreviewing = ispreview;
    }

    /**
     * Get flag which indicates whether current status is previewing mode.
     * 
     * @return Return the previewing flag.
     */
    public boolean getPreviewFlag() {
        return isPreviewing;
    }

    /**
     * Return the bitmap object which will be passed in
     * {@link ImagingManager#getSingleFrame} and then will contain the captured
     * image data if successfully.
     * 
     * @return Return the bitmap object for capturing.
     */
    public Bitmap getCaptureBitmap() {
        // use this interface to normal base
        if (mImageFileBitmap != null) {
            mImageFileBitmap.recycle();
            mImageFileBitmap = null;
            isNeedShowImagefile = false;
        }
        return mBitmap;
    }

    public void setBitmap(Bitmap bitmap) {
        mBitmap = bitmap;
    }

    public void setPreviewBitmap(Bitmap bitmap) {
        mPreviewBitmap = bitmap;
    }

    public void freePreviewBitmap() {
        if (mPreviewBitmap != null && !mPreviewBitmap.isRecycled()) {
            mPreviewBitmap.recycle();
            mPreviewBitmap = null;
        }
    }

    public void creatPreviewBitmap() {
//        if (mPreviewBitmap == null) {
//            mPreviewBitmap = Bitmap.createBitmap(CAPTUERED_IMAGE_WIDTH / PREVIEW_RATIO,
//                    CAPTUERED_IMAGE_HEIGHT / PREVIEW_RATIO, Bitmap.Config.ALPHA_8);
//        }
    }

    /**
     * Return the bitmap object which will be passed in
     * {@link ImagingManager#getPreviewingFrame} and then will contain the
     * previewing image data .
     * 
     * @return Return the bitmap object for previewing.
     */
    public Bitmap getPreviewBitmap() {
        // use this interface to normal base
        if (mImageFileBitmap != null) {
            mImageFileBitmap.recycle();
            mImageFileBitmap = null;
        }
        return mPreviewBitmap;
    }

    public int getZoomPer() {
        if (mOrginWidth != 0)
            return mScaleWidht * 100 / mOrginWidth;
        return -1;
    }

    /**
     * Return the string containing the configure of the image.
     * 
     * @return Return string object.
     */
    public String getImagedepth() {
        if (mImageFileBitmap != null) {
            return mImageFileBitmap.getConfig().name();
        } else {
            return mBitmap.getConfig().name();
        }
    }

    /**
     * Return the string containing the resolution of the image.
     * 
     * @return Return string object.
     */
    public String getResolution() {
        if (mImageFileBitmap != null)
            return mImageFileBitmap.getWidth() + "x" + mImageFileBitmap.getHeight();
        else
            return mBitmap.getWidth() + "x" + mBitmap.getHeight();
    }

    /**
     * Creates the bitmap object decoded form file path of {@link filename} and
     * configure corresponding layout.
     * 
     * @param filename The absolute path for selected image file.
     * @return Whether this method execute successfully.
     */
    public boolean setBitmapByFN(String filename) {
        BitmapFactory.Options option = new BitmapFactory.Options();
        option.inSampleSize = 1;
        if (mImageFileBitmap != null)
            mImageFileBitmap.recycle();
        mImageFileBitmap = BitmapFactory.decodeFile(filename, option);
        if (mImageFileBitmap != null) {
            int w = mImageFileBitmap.getWidth() > getWidth() ? getWidth() : mImageFileBitmap
                    .getWidth();
            int h = mImageFileBitmap.getHeight() > getHeight() ? getHeight() : mImageFileBitmap
                    .getHeight();
            mMinWidth = w / 4;
            mMaxWidth = w * 2;
            mWidhtStep = w * 1000 / 40;
            mHeightStep = h * 1000 / 40;
            mOrginWidth = mScaleWidht = w;
            mOriginHeight = mScaleHeight = h;
            mZoomNum = 0;
            isNeedShowImagefile = true;
            setShowBitmapflag(false);
            return true;
        }

        return false;
    }

    /**
     * Set the handler ,which is in charge of refreshing the screen of user's
     * activity, to ImagingView .
     * 
     * @param handler The Handler object from user's activity.
     */
    public void setHandler(Handler handler) {
        this.mHandler = handler;
    }

    /**
     * Disconnects from engine and releases the bitmap buffer in ImagingView
     * ,such as captured image,previewing frame,etc . This method is invoked
     * when user need to exit his activity.
     */
    public void stop() {
    	if (mBitmap != null && !mBitmap.isRecycled()){
        mBitmap.recycle();
        mBitmap = null;
    	}

        if (mImageFileBitmap != null) {
            mImageFileBitmap.recycle();
            mImageFileBitmap = null;
        }

        if (mPreviewBitmap != null) {
            mPreviewBitmap.recycle();
            mPreviewBitmap = null;
        }
        isPreviewing = false;
    }

    /**
     * Zooms in or zooms out image.
     * 
     * @param iszoomout Whether is to zoom out image.
     */
    public void zoomCanvas(boolean iszoomout) {
        if (iszoomout) {
            if (mScaleWidht > mMinWidth) {

                mZoomNum--;
                mScaleWidht = mOrginWidth + (mWidhtStep * mZoomNum) / 1000;
                mScaleHeight = mOriginHeight + (mHeightStep * mZoomNum) / 1000;
            }
        } else {
            // zoom in
            if (mScaleWidht < mMaxWidth) {
                mZoomNum++;
                mScaleWidht = mOrginWidth + (mWidhtStep * mZoomNum) / 1000;
                mScaleHeight = mOriginHeight + (mHeightStep * mZoomNum) / 1000;
            }
        }

        invalidate();
        mHandler.sendEmptyMessage(MESSAGE_UPDATE_ZOOMBUTTON);
    }

    /**
     * Determine whether zoom buttons are enabled to click.
     * 
     * @param iszoomout Whether the selected button is zoom out button
     * @return Returns the enable status of current zoom button.
     */
    public boolean isZoomEnabled(boolean isZoomout) {
        boolean result = true;

        if (!isNeedShowImagefile && !isNeedshowBitmap && !isPreviewing)
            return false;
        if (isZoomout) {
            if (mScaleWidht <= mMinWidth) {
                result = false;
            }
        } else {
            if (mScaleWidht >= mMaxWidth) {
                result = false;
            }
        }

        return result;
    }

    @Override
    protected void onDraw(Canvas canvas) {
        // for the case of showing image by filename
        if (isNeedUpdateLayout) {
            initCaptureParametre();
            isNeedUpdateLayout = false;
        }
        if (isPreviewing && mPreviewBitmap != null && !mPreviewBitmap.isRecycled()) {
            Paint p = new Paint();
            p.setColor(Color.WHITE);
            Rect mSrc = new Rect();
            mSrc.set(0, 0, mPreviewBitmap.getWidth(), mPreviewBitmap.getHeight());
            Rect mDst = new Rect();
            mDst.set(0, 0, mScaleWidht, mScaleHeight);
            canvas.drawBitmap(mPreviewBitmap, null, mDst, p);

        } else {
            if (mImageFileBitmap != null) {
                isNeedshowBitmap = false;
                canvas.drawColor(Color.BLUE);
                Rect mSrc = new Rect();
                mSrc.set(0, 0, mImageFileBitmap.getWidth(), mImageFileBitmap.getHeight());
                Rect mDst = new Rect();
                mDst.set(0, 0, mScaleWidht, mScaleHeight);
                canvas.drawBitmap(mImageFileBitmap, mSrc, mDst, null);
            } else if (mBitmap != null) {
                if (isNeedshowBitmap) {
                    this.setBackgroundColor(Color.BLACK);
                    Rect mSrc = new Rect();
                    canvas.drawColor(Color.BLUE);
                    mSrc.set(0, 0, mBitmap.getWidth(), mBitmap.getHeight());
                    Rect mDst = new Rect();
                    mDst.set(0, 0, mScaleWidht, mScaleHeight);
                    canvas.drawBitmap(mBitmap, mSrc, mDst, null);
                }

            } else {

            }
        }

    }

}
