
package com.honeywell.samples.imagingsample;

import com.honeywell.decodemanager.barcode.BitmapParcel;
import com.honeywell.imagingmanager.ImageConst;
import com.honeywell.imagingmanager.ImagingManager;
import com.honeywell.iqimagemanager.IQConstValue;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.graphics.Bitmap;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.os.SystemClock;
import android.util.Log;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.LinearLayout;
import android.widget.Toast;

import java.io.File;
import java.io.IOException;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

public class ImagingSample extends Activity implements IQConstValue,IConsteValue,
        ShortcutButton.OnShortcutButtonListener, ImageConst {
    private ImagingView mImageView;

    private MenuInflater mInflater;

    static final String TAG = "ImagingSample";

    private boolean isZoomBarShowing = false;

    private boolean isAutoCaputreOn = false;
    
    private boolean mAutoCaptureThread = false;

    private ImagingManager mImagingManager;

    private LinearLayout mButtonlayout;

    private ShortcutButton mZoominButton;

    private ShortcutButton mZoomoutButton;

    private ProgressDialog mSavingProgress;

    private Context mContext;
    
    private BitmapParcel mFrameParcel;

    private BitmapParcel mPreviewParcel;

    private Lock mImagingServiceLock = new ReentrantLock();
    
    private int mOperationType;
    
    private long starttime = 0;
    
    private int mScanOnTimer = 0;

    private Bitmap mCaptureBmp = null;

    private long mPreviewingTimer = 500;
    
    private final int SCANKEY = 0x94;
    
    private final Runnable mRunnable = new Runnable() {
        public void run() {
            
            // previewing
            if (!isAutoCaputreOn) {
                isAutoCaputreOn = true;
                autoCaputureFrame(true);
            }
        }
    };

    private Handler mHandler = new Handler() {
        public void handleMessage(Message msg) {
            switch (msg.what) {
                
                case MESSAGE_READY_DOCODER:
    				int width = mImagingManager.getImageWidth();
    				int height = mImagingManager.getImageHeight();
    				int id = mImagingManager.getEngineID();
    				mImageView.initBitmap(width, height);
    				if (mCaptureBmp == null)
    					mCaptureBmp = Bitmap.createBitmap(width, height,Bitmap.Config.ALPHA_8);
                    if (isAutoCaputreOn)
                        mImagingManager.sendImageRequest(OPERATION_PREVIEWING);
                    break;
                case MESSAGE_READY_TO_IMAGE:// image process ,need add types for                                            // frame and preview
                    captureFrame();
                    break;
                case MESSAGE_READY_TO_PREVIEW:
                    if (isAutoCaputreOn)
                        autoCaputureFrame(true);
                    break;
                case MESSAGE_UPDATE_IMG:
                    SoundManager.playSound(1, 1);
                    mImageView.invalidate();
                    updateZoomBuuton();
                    break;

                case MESSAGE_UPDATE_PREVEIW:
                    mImageView.setBackgroundDrawable(null);
                    mImageView.invalidate();
                    if (isAutoCaputreOn)
                        mImagingManager.sendImageRequest(OPERATION_PREVIEWING);
                    break;

                case MESSAGE_UPDATE_ZOOMBAR:
                    mButtonlayout.setVisibility(isZoomBarShowing ? View.VISIBLE : View.GONE);
                    mImageView.setUpdateFlag(true);
                    break;

                case MESSAGE_UPDATE_ZOOMBUTTON:
                    updateZoomBuuton();
                    break;

                case MESSAGE_PREVIEWING:
                    doWithAutoCapture(true);
                    break;

                case MESSAGE_BEGIN_SAVE:
                    if (!mImagingManager.saveBitmap("", IMAGE_PNG, mImageView.getCaptureBitmap()))
                        showWarningDialog(R.string.app_name, R.string.save_error, null);
                    else {
                        mSavingProgress.dismiss();
                        Toast.makeText(mContext, mContext.getText(R.string.save_success),
                                Toast.LENGTH_LONG).show();
                    }
                    break;

                default:
                    break;
            }
        }
    };

    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        mContext = this;
        mImageView = (ImagingView) findViewById(R.id.demoview);
        if (!isAutoCaputreOn)
            mImageView.setBackgroundResource(R.drawable.background);
        mImageView.setHandler(mHandler);

        mButtonlayout = (LinearLayout) findViewById(R.id.zoombar);
        mZoominButton = (ShortcutButton) findViewById(R.id.zoomin);
        mZoominButton.setOnShortcutButtonListener(this);
        mZoomoutButton = (ShortcutButton) findViewById(R.id.zoomout);
        mZoomoutButton.setOnShortcutButtonListener(this);
        SoundManager.getInstance();
        SoundManager.initSounds(this);
        SoundManager.loadSounds();
        mImagingManager = new ImagingManager(this, mHandler);
        mSavingProgress = new ProgressDialog(this);
        mAutoCaptureThread = isAutoCaputreOn;
    }

    @Override
    public void onResume() {
        super.onResume();
        File file = new File(STRING_IMAGE_FOLDER);
        if (!file.isDirectory()) {
            file.mkdir();
        }
        if (isZoomBarShowing) {
            mButtonlayout.setVisibility(View.VISIBLE);
        }
        updateZoomBuuton();

        if (isAutoCaputreOn) {
            mHandler.sendEmptyMessageDelayed(MESSAGE_PREVIEWING, 50);
        }
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        if (mImageView != null) {
            mImageView.stop();
            mImageView = null;
        }
        mHandler.removeCallbacksAndMessages(null);
        // unbindImagingService();
        SoundManager.cleanup();
    }

    @Override
    public void onPause() {
        super.onPause();
        // if is paused ,need set auto to false
        if (isAutoCaputreOn) {
            autoCaputureFrame(false);
        }

        try {
            mImagingManager.release();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        mSavingProgress.dismiss();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        mInflater = getMenuInflater();
        mInflater.inflate(R.menu.image_operation, menu);
        return true;
    }

    @Override
    public boolean onPrepareOptionsMenu(Menu menu) {
        if (!isAutoCaputreOn && mImageView.isNeedShowBitmap())
            menu.getItem(1).setEnabled(true);
        else
            menu.getItem(1).setEnabled(false);

        if (isAutoCaputreOn) {
            menu.getItem(0).setTitle(getText(R.string.menu_preview_off));

        } else {
            menu.getItem(0).setTitle(getText(R.string.menu_preview));

        }

        if (isZoomBarShowing)
            menu.getItem(2).setTitle(getText(R.string.menu_zoom_off));
        else
            menu.getItem(2).setTitle(getText(R.string.menu_zoom));

        return super.onPrepareOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle item selection
        switch (item.getItemId()) {
            case R.id.menu_preview:
                isAutoCaputreOn = !isAutoCaputreOn;
                doWithAutoCapture(isAutoCaputreOn);
                return true;
            case R.id.menu_save:
                saveImage();
                return true;
            case R.id.menu_zoom:
                isZoomBarShowing = !isZoomBarShowing;
                mHandler.sendEmptyMessage(MESSAGE_UPDATE_ZOOMBAR);
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }
    
    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        switch (keyCode) {
            case 92: // scankey
            //case 292:
                //if (!mAutoCaptureThread){
               //     mHandler.postDelayed(mRunnable, mPreviewingTimer);
               // }
                event.startTracking();
                // isAutoCaputreOn = false;
                // scanImage();
                return true;
            case KeyEvent.KEYCODE_BACK:
                this.finish();
                return true;
            case KeyEvent.KEYCODE_UNKNOWN:	
            	if(event.getScanCode() == 0x94 || event.getScanCode() == 87 || event.getScanCode() == 88) {
            		event.startTracking();
            	}
            	return true;
            default:
                return super.onKeyDown(keyCode, event);
        }
    }

    @Override
    public boolean onKeyLongPress(int keyCode, KeyEvent event) {
        switch (keyCode) {
            case KeyEvent.KEYCODE_ENTER:
            case 92: // scankey
            //case 292:
                if (!mAutoCaptureThread) {
                	mAutoCaptureThread = true;
                    mImagingManager.sendImageRequest(OPERATION_PREVIEWING);
                }

                return true;
                
            case KeyEvent.KEYCODE_UNKNOWN:	
            	if(event.getScanCode() == 0x94  || event.getScanCode() == 87 || event.getScanCode() == 88) {
                    if (!mAutoCaptureThread) {
                    	mAutoCaptureThread = true;
                        mImagingManager.sendImageRequest(OPERATION_PREVIEWING);
                    }
            	}
            	return true;
            default:
                return super.onKeyLongPress(keyCode, event);
        }
    }

    @Override
    public boolean onKeyUp(int keyCode, KeyEvent event) {
        switch (keyCode) {
            case KeyEvent.KEYCODE_ENTER:
            case 92: // scankey
            //case 292:
               // mHandler.removeCallbacks(mRunnable);
                if (mAutoCaptureThread) {
                    autoCaputureFrame(false);
                }
                scanImage();
                return true;
                
            case KeyEvent.KEYCODE_UNKNOWN:	
            	if(event.getScanCode() == 0x94 || event.getScanCode() == 87 || event.getScanCode() == 88) {
                    if (mAutoCaptureThread) {
                        autoCaputureFrame(false);
                    }
                    scanImage();
            	}
            	return true;
            default:
                return super.onKeyUp(keyCode, event);
        }
    }

    @Override
    public void onShortcutButtonClick(ShortcutButton button) {
        // TODO Auto-generated method stub
        switch (button.getId()) {
            case R.id.zoomin:
                mImageView.zoomCanvas(false);
                break;
            case R.id.zoomout:
                mImageView.zoomCanvas(true);
                break;
            default:
                break;
        }
    }

    private void imageProcessImage(int type) {
        mOperationType = type;
        // TODO Auto-generated method stub

        switch (mOperationType) {
            case OPERATION_FRAME:
                mImagingManager.ImageProcessingFrame(mFrameParcel.getBitmap(),
                        mImageView.getCaptureBitmap(), OPERATION_FRAME);

                mHandler.sendEmptyMessage(MESSAGE_UPDATE_IMG);
                break;
            case OPERATION_PREVIEWING:
                mImagingManager.ImageProcessingFrame(mPreviewParcel.getBitmap(),
                        mImageView.getPreviewBitmap(), OPERATION_PREVIEWING);
                // mImageView.setPreviewBitmap(mFrameParcel.getBitmap());
                mHandler.sendEmptyMessage(MESSAGE_UPDATE_PREVEIW);
                break;
            default:
                break;
        }
    }
    private class PreviewRunnable implements Runnable {
        @Override
        public void run() {
            int ret;
            mImagingServiceLock.lock();
            if (mImageView.getPreviewBitmap()!= null) {
                mPreviewParcel = new BitmapParcel(mImageView.getPreviewBitmap());
                ret = mImagingManager.doImageAction(mPreviewParcel, OPERATION_PREVIEWING);
                if (ret != ImagingResult_Succeed){
                    // if get image from decoder service fails, exit this procedure
                    Log.d(TAG,"Error in getting preview from decoder");
                }else{
                    imageProcessImage(OPERATION_PREVIEWING);
                }
                mPreviewParcel.releaseBitmap();
                mPreviewParcel = null;
                
            }
            mImagingServiceLock.unlock();
            Log.d(TAG, "Get and process previewing cost" + (SystemClock.uptimeMillis() - starttime)
                    + " ms");
        }
    }

    private class FrameRunnable implements Runnable {
        @Override
        public void run() {
            mImagingServiceLock.lock();
            if (mCaptureBmp != null) {
                int ret;
                mImagingManager.startScanning();
                mFrameParcel = new BitmapParcel(mCaptureBmp);
                // (new Thread(new PreviewRunnable(mBitmap))).start();
                ret = mImagingManager.doImageAction(mFrameParcel, OPERATION_FRAME);
                mImagingManager.stopScanning();
                if (ret != ImagingResult_Succeed)
                    Log.d(TAG,"Error in capturing from decoder");
                else
                    imageProcessImage(OPERATION_FRAME);
                mImageView.setPreviewFlag(false);
                mFrameParcel.releaseBitmap();
                mFrameParcel = null;
            }
            mImagingServiceLock.unlock();
        }
    }

    private void captureFrame() {
        mImageView.setShowBitmapflag(true);
        (new Thread(new FrameRunnable())).start();
    }

    private void autoCaputureFrame(boolean isPreviewing) {

        if (isPreviewing){
            if (mScanOnTimer<=0){
                mImagingManager.startScanning(); 
                mScanOnTimer++;
            }                
        }else {
            if (mScanOnTimer>0){
                mImagingManager.stopScanning();
                mScanOnTimer--;
            }
                
        }
        isAutoCaputreOn = isPreviewing;
        mImageView.setPreviewFlag(isPreviewing);

        if (isPreviewing) {
            starttime = SystemClock.uptimeMillis();
            (new Thread(new PreviewRunnable())).start();
        }

    }



    private void updateZoomBuuton() {

        // update zoomout
        boolean isenable = mImageView.isZoomEnabled(true);
        mZoomoutButton.setImageResource(isenable ? R.drawable.zoomout : R.drawable.zoomout_dis);
        mZoomoutButton.setClickable(isenable);
        // update zoomin
        boolean otherable = mImageView.isZoomEnabled(false);
        mZoominButton.setImageResource(otherable ? R.drawable.zoomin : R.drawable.zoomin_dis);
        mZoominButton.setClickable(otherable);

    }

    private void doWithAutoCapture(boolean isON) {
        if (mImageView != null) {
            autoCaputureFrame(isAutoCaputreOn);
            if (!isAutoCaputreOn)
                scanImage();
        }
    }

    private void scanImage() {
        mImagingManager.sendImageRequest(OPERATION_FRAME);
    }

    private boolean isSDExists() {
        boolean result = false;
        result = (android.os.Environment.getExternalStorageState()
                .equals(android.os.Environment.MEDIA_MOUNTED));
        if (!result) {
            showWarningDialog(R.string.app_name, R.string.sd_not_exist, null);
        }
        return result;
    }

    private boolean isCaptured() {
        boolean result = mImageView.isNeedShowBitmap();
        if (!result)
            showWarningDialog(R.string.app_name, R.string.not_capture, null);
        return result;
    }

    private void showWarningDialog(int titleid, int messgeid, String message) {
        AlertDialog dialog = new AlertDialog.Builder(this)
                .setIcon(android.R.drawable.ic_dialog_alert)
                .setMessage((messgeid == 0) ? message : getString(messgeid)).setTitle(titleid)
                .setCancelable(false)
                .setPositiveButton(android.R.string.ok, new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int id) {
                        dialog.dismiss();
                    }
                }).create();

        dialog.show();
    }

    private void saveImage() {
        if (isCaptured()) {
            if (isSDExists()) {
                mSavingProgress.setIndeterminate(true);
                mSavingProgress.setMessage(getText(R.string.save_process));
                mSavingProgress.show();
                mHandler.sendEmptyMessageDelayed(MESSAGE_BEGIN_SAVE, 500);
            }
        }
    }
}