/*
 * Copyright (C) 2008 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.honeywell.samples.imagingsample;

import android.content.Context;
import android.util.AttributeSet;
import android.widget.ImageView;

/**
 * A button designed to be used for the on-screen shutter button. It's currently
 * an ImageView that can call a delegate when the pressed state changes.
 */
public class ShortcutButton extends ImageView implements IConsteValue {
    /**
     * Interface definition for a callback to be invoked when a ModeButton's
     * pressed state changes.
     */
    public interface OnShortcutButtonListener {
        void onShortcutButtonClick(ShortcutButton b);
    }

    private OnShortcutButtonListener mListener;

    public ShortcutButton(Context context) {
        super(context);
        this.setBackgroundDrawable(context.getResources().getDrawable(R.drawable.button_backgroud));
    }

    public ShortcutButton(Context context, AttributeSet attrs) {
        super(context, attrs);
        this.setBackgroundDrawable(context.getResources().getDrawable(R.drawable.button_backgroud));
    }

    public ShortcutButton(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
        this.setBackgroundDrawable(context.getResources().getDrawable(R.drawable.button_backgroud));
    }

    public void setOnShortcutButtonListener(OnShortcutButtonListener listener) {
        mListener = listener;
    }

    /**
     * Hook into the drawable state changing to get changes to isPressed -- the
     * onPressed listener doesn't always get called when the pressed state
     * changes.
     */
    @Override
    protected void drawableStateChanged() {
        super.drawableStateChanged();
    }

    @Override
    public boolean performClick() {
        boolean result = super.performClick();
        if (mListener != null) {
            mListener.onShortcutButtonClick(this);
        }
        return result;
    }

}
