/*======================================================================

    UNLESS OTHERWISE AGREED TO IN A SIGNED WRITING BY HONEYWELL INTERNATIONAL INC
    (HONEYWELL) AND THE USER OF THIS CODE, THIS CODE AND INFORMATION IS PROVIDED
    "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING
    BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS
    FOR A PARTICULAR PURPOSE.

    COPYRIGHT (C) 2011 HONEYWELL INTERNATIONAL INC.

    THIS SOFTWARE IS PROTECTED BY COPYRIGHT LAWS OF THE UNITED STATES OF
    AMERICA AND OF FOREIGN COUNTRIES. THIS SOFTWARE IS FURNISHED UNDER A
    LICENSE AND/OR A NONDISCLOSURE AGREEMENT AND MAY BE USED IN ACCORDANCE
    WITH THE TERMS OF THOSE AGREEMENTS. UNAUTHORIZED REPRODUCTION,  DUPLICATION
    OR DISTRIBUTION OF THIS SOFTWARE, OR ANY PORTION OF IT  WILL BE PROSECUTED
    TO THE MAXIMUM EXTENT POSSIBLE UNDER THE LAW.

======================================================================*/

package com.honeywell.samples.ioctlsample;

import com.honeywell.ioctl.HoneywellIOCTL;
import com.honeywell.ioctl.IoctlRWStruct;

import android.app.Activity;
import android.os.Bundle;
import android.text.method.ScrollingMovementMethod;
import android.util.Log;
import android.widget.TextView;

public class IoctlSampleActivity extends Activity {
    private final String TAG = "IoctlSample";
    private TextView tv;
    
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        //setContentView(R.layout.main);
        
        tv = new TextView(this);
        tv.setMovementMethod(new ScrollingMovementMethod());
       
        showInfo();
        setContentView(tv);
        
    }
    
    private void showInfo()
    {
        String result = "SYSTEM INFO:\n";
        result += "-------------------------\n";
        
        result += "XLOADER_VERSION: " + readHsmVersionInfo(HoneywellIOCTL.HSM_XLOADER_VERSION) + "\n"
            + "UBOOT_VERSION: " + readHsmVersionInfo(HoneywellIOCTL.HSM_UBOOT_VERSION) + "\n"
            + "RECOVERY_VERSION: " + readHsmVersionInfo(HoneywellIOCTL.HSM_RECOVERY_VERSION) + "\n"
            + "OSIMAGE_VERSION: " + readHsmVersionInfo(HoneywellIOCTL.HSM_OSIMAGE_VERSION) + "\n"
            + "MCU_VERSION: " + readHsmVersionInfo(HoneywellIOCTL.HSM_MCU_VERSION) + "\n"
            + "MFG_DATE: " + readHsmVersionInfo(HoneywellIOCTL.HSM_MFG_DATE) + "\n"
            + "MFG_MODEL_NUM: " + readHsmVersionInfo(HoneywellIOCTL.HSM_MFG_MODEL_NUM) + "\n"
            + "MFG_SERIAL_NUM: " + readHsmVersionInfo(HoneywellIOCTL.HSM_MFG_SERIAL_NUM) + "\n"
            + "MFG_PART_NUM: " + readHsmVersionInfo(HoneywellIOCTL.HSM_MFG_PART_NUM) + "\n"
            + "ODM_TRACKING_NUM: " + readHsmVersionInfo(HoneywellIOCTL.HSM_ODM_TRACKING_NUM) + "\n"
            + "CONFIG_NUM: " + readHsmVersionInfo(HoneywellIOCTL.HSM_CONFIG_NUM) + "\n"
            + "CUSTOM_SERIAL_NUM: " + readHsmVersionInfo(HoneywellIOCTL.HSM_CUSTOM_SERIAL_NUM) + "\n"
            + "FLASH_SIZE: " + readHsmVersionInfo(HoneywellIOCTL.HSM_FLASH_SIZE) + "\n"
            + "KEYBOARD_TYPE: " + readHsmVersionInfo(HoneywellIOCTL.HSM_KEYBOARD_TYPE) + "\n"
            + "HARDWARE_REV: " + readHsmVersionInfo(HoneywellIOCTL.HSM_HARDWARE_REV) + "\n"
            + "FEATURE_SET: " + readHsmVersionInfo(HoneywellIOCTL.HSM_FEATURE_SET) + "\n"
            + "RESET_REASON: " + readHsmVersionInfo(HoneywellIOCTL.HSM_RESET_REASON) + "\n"
            + "WAKE_SOURCE: " + readHsmVersionInfo(HoneywellIOCTL.HSM_WAKE_SOURCE) + "\n"
            + "MFG_DATA_VALID: " + readHsmVersionInfo(HoneywellIOCTL.HSM_MFG_DATA_VALID) + "\n"; 
        
        result += "-------------------------\n";
        
        this.tv.setText(result); 
    }
    
    private int byteArrayToInt(byte[] b) throws Exception
    {
        if (b != null && b.length >= 4) {
          return ((int)(b[3] & 0xff) << 24) 
               | ((int)(b[2] & 0xff) << 16) 
               | ((int)(b[1] & 0xff) << 8 ) 
               | ((int)(b[0] & 0xff) << 0 );  
        } else
            throw new Exception();
    }
    
    private String readHsmVersionInfo(int id) {
        String result = "";

        HoneywellIOCTL ioctl = new HoneywellIOCTL();

        IoctlRWStruct data = new IoctlRWStruct();
        data.dataItem = id;
        data.buffer = new byte[1024];
        data.bufferSize = data.buffer.length;

        switch (id) {
            // String
            case HoneywellIOCTL.HSM_XLOADER_VERSION:
            case HoneywellIOCTL.HSM_UBOOT_VERSION:
            case HoneywellIOCTL.HSM_RECOVERY_VERSION:
            case HoneywellIOCTL.HSM_OSIMAGE_VERSION:
            case HoneywellIOCTL.HSM_MCU_VERSION:
            case HoneywellIOCTL.HSM_BATTERY_SERIAL_NUM:
            case HoneywellIOCTL.HSM_MFG_DATE:
            case HoneywellIOCTL.HSM_MFG_MODEL_NUM:
            case HoneywellIOCTL.HSM_MFG_SERIAL_NUM:
            case HoneywellIOCTL.HSM_MFG_PART_NUM:
            case HoneywellIOCTL.HSM_ODM_TRACKING_NUM:
            case HoneywellIOCTL.HSM_CONFIG_NUM:
            case HoneywellIOCTL.HSM_CUSTOM_SERIAL_NUM:
                if (ioctl.hsmReadData(data) == 0 && data.readSize > 0) {
                    result = new String(data.buffer, 0, data.readSize);
                }
                break;
            // Integer
            case HoneywellIOCTL.HSM_FLASH_SIZE:
            case HoneywellIOCTL.HSM_KEYBOARD_TYPE:
            case HoneywellIOCTL.HSM_HARDWARE_REV:
            case HoneywellIOCTL.HSM_FEATURE_SET:
            case HoneywellIOCTL.HSM_RESET_REASON:
            case HoneywellIOCTL.HSM_WAKE_SOURCE:
            case HoneywellIOCTL.HSM_MFG_DATA_VALID:
                if (ioctl.hsmReadData(data) == 0 && data.readSize > 0) {
                    try {                        
						result = "0x"+Integer.toHexString(byteArrayToInt(data.buffer));
                    } catch (Exception e) {
                        // TODO Auto-generated catch block
                        e.printStackTrace();
                    }                    
                }
                break;
            default:
                break;
        }

        Log.d(TAG, result);
        return result;
    }
}